<?php

namespace MyMenu\Services;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

use Kreait\Firebase;

class FCMService {
    const DEFAULT_FILE_NAME = 'tecfood-mymenu-firebase-adminsdk.json';
    protected $defaultFilePath;

    public function __construct(){
        $this->defaultFilePath = realpath(implode(DIRECTORY_SEPARATOR, ['..', 'config', self::DEFAULT_FILE_NAME]));
    }

    public function getServiceAccountPath(){
        return $this->defaultFilePath;
    }

    private function getMessaging() {
        try {
            $serviceAccountPath = $this->getServiceAccountPath();
            $firebase = (new Firebase\Factory)->withServiceAccount($serviceAccountPath);
            $messaging = $firebase->createMessaging();
            return $messaging;
        } catch(\Exception $e) {
            throw new \Exception("ERR_CREATE_MESSAGING$$".$e->getMessage());
        }
    }

    public function subscribeToTopic( $topic, $token ){
        if( !$token || !$topic ) {
            throw new \Exception("ERR_INVALID_PARAMS");
        }

        $messaging = $this->getMessaging();

        try {
            $messaging->subscribeToTopic($topic, $token);
        } catch( \Exception $e ) {
            throw new \Exception("ERR_UNEXPECTED_ERROR_SUBSCRIBE_TOPIC$$".$e->getMessage());
        }
    }
    
    public function unsubscribeFromAllTopics( $token ) {
        if ( !$token ) {
            throw new \Exception("ERR_INVALID_PARAMS");
        }
        
        $messaging = $this->getMessaging();
        
        try {
            $messaging->unsubscribeFromAllTopics( $token );
        } catch( \Exception $e ) {
            throw new \Exception("ERR_UNEXPECTED_ERROR_UNSUBSCRIBE_TOPIC$$".$e->getMessage());
        }
    }
}