module.exports = function (grunt) {

    var backendModules = [];
    var tecfood = {
        js: [
            "./assets/js/util/**/*.js",
            "./assets/js/services/**/*.js",
            "./assets/js/controllers/**/*.js",
            "./assets/templates/**/*.js",
            "!./assets/js/libs.js",
            "!./assets/js/engine.js",
            "!./assets/js/environment.js",
            "!./assets/js/app.js",
            "!./assets/js/app.min.js"
        ],
        outterjs: [
            // "bower_components/red-ui/dist/rui.js",
            "bower_components/red-ui-fields/dist/rui-fields.js",
            "bower_components/red-ui-footer/dist/rui-footer.js",
            "bower_components/red-ui-header/dist/rui-header.js",
            "bower_components/red-ui-list/dist/rui-list.js",
            "bower_components/red-ui-sidenav/dist/rui-sidenav.js"
        ],
        less: ['assets/less/**/*.less', 'assets/less/*.less'],
        css: ["./assets/css/**/*.css", "./assets/templates/**/*.css", "!./assets/css/tecfood-quest.css", "!./assets/css/app.css", "!./assets/css/app.min.css", "!./assets/css/css-compile.css"],
        libs: [
            "./bower_components/zh-customizations-service/dist/zhCustomizationsService.js",
            "./bower_components/zh-filter-preferences/dist/zhFilterPreferences.js",
            "./bower_components/zh-preferences/dist/preferences.js",
            "./bower_components/zhuaclfrontend/assets/dist/main.js",
            "./bower_components/zhulog/dist/main.js",
            "./bower_components/tek-libraries/dist/main.js"
        ],
        outtercss: [
            "bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css",
            "bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css",
            "bower_components/zeedhi-frontend/assets/css/styles/styles-tecfood.css",
            "bower_components/red-ui/src/assets/fonts/Rui-Icons/rui-icons.css",
            "bower_components/red-ui/src/assets/fonts/Roboto/Rb-style.css",
            "bower_components/red-ui/dist/rui.css",
            "bower_components/red-ui-fields/dist/rui-fields.css",
            "bower_components/red-ui-footer/dist/rui-footer.css",
            "bower_components/red-ui-header/dist/rui-header.css",
            "bower_components/red-ui-list/dist/rui-list.css",
            "bower_components/red-ui-sidenav/dist/rui-sidenav.css",
            "bower_components/zh-customizations-service/dist/style/style.css"
        ]
    };

    grunt.initConfig({
        run: {
            firebase_build: {
                exec: 'npm run build'
            }   
        },
        concat: {
            js: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: tecfood.js,
                dest: 'assets/js/app.js'
            },
            outterjs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: tecfood.outterjs,
                dest: 'assets/js/outterjs.js'
            },
            libs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: tecfood.libs,
                dest: 'assets/js/libs.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: tecfood.css,
                dest: 'assets/css/app.css'
            },
            outtercss: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: tecfood.outtercss,
                dest: 'assets/css/outtercss.css'
            }
        },
        less: {
            development: {
                options: {
                    compress: true,
                    modifyVars: {
                        fontFamily: 'Roboto, san serif',
                        accentColor: '#CA3836'
                    }
                },
                files: {
                    'assets/css/css-compile.css': "assets/less/import-ordering.less"
                }
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/js/app.min.js': tecfood.js
                }
            }
        },
        jshint: {
            all: tecfood.js.concat(["GruntFile.js"]),
            options: {
                evil: true,
                esversion: 9
            }
        },
        watch: {
            js: {
                files: tecfood.js.concat(["GruntFile.js"]),
                tasks: ['jshint', 'concat:js']
            },
            css: {
                files: tecfood.css,
                tasks: ['concat:css']
            },
            less: {
                files: tecfood.less,
                tasks: ['less']
            }
        },
        cssmin: {
            dist: {
                src: tecfood.css,
                dest: 'assets/css/app.min.css'
            }
        },
        // NÂO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        zhIdGenerator: {
            options: {
                jsonPath: ["./assets/json"],
                menuPath: ["./assets/json/menu.json"]
            }
        },
        buildAclStruct: {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
            development: {
                configFile: "config/development.json"
            },
            demo: {
                configFile: "config/demo.json"
            },
            production: {
                configFile: "config/deploy.json"
            },
            test: {
                configFile: "config/test.json"
            },
            srcFolders: ["../backend/config/", "./assets/js", "./assets/json", "./assets/json/words", "./config", "./", "./assets"]
        },
        zhWords: {
            options: {
                serverUrl: "https://translate.teknisa.com/backend/index.php",
                organization: "20",
                project: "10914"
            }
        }
    });

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zh-words');
    grunt.loadNpmTasks('grunt-run');

    grunt.registerTask('minify', ['cssmin', 'uglify']);
    grunt.registerTask('dist', ['less', 'concat', 'minify', 'buildAclStruct', 'zhWords']);
    
    // Tasks pre building apps
    grunt.registerTask('prod', ['dist', 'jshint', 'buildEnvironment:production']);
    grunt.registerTask('debug', ['dist', 'buildEnvironment']);

    grunt.registerTask('default', ['run:firebase_build', 'less', 'jshint', 'concat', 'zhWords', 'buildEnvironment']);
};